@@ -13,7 +13,6 @@ use super::constants::Tag;
 use super::super::{
     CompoundClass,
     DomainDecode,
-    Embeddable,
     Map,
     NestedValue,
     Record,
@@ -31,32 +30,38 @@ use super::super::{
     signed_integer::SignedInteger,
 };
 
-pub struct PackedReader<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: BinarySource<'de>> {
+pub struct PackedReader<'de, 'src, N: NestedValue, Dec: DomainDecode<N::Embedded>, S: BinarySource<'de>> {
     pub source: &'src mut S,
     pub decode_embedded: Dec,
-    phantom: PhantomData<&'de (D, N)>,
+    phantom: PhantomData<&'de N>,
 }
 
-impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: BinarySource<'de>>
+impl<'de, 'src, N: NestedValue, Dec: DomainDecode<N::Embedded>, S: BinarySource<'de>>
     BinarySource<'de>
-    for PackedReader<'de, 'src, D, N, Dec, S>
+    for PackedReader<'de, 'src, N, Dec, S>
 {
     type Mark = S::Mark;
+    #[inline(always)]
     fn mark(&mut self) -> io::Result<Self::Mark> {
         self.source.mark()
     }
+    #[inline(always)]
     fn restore(&mut self, mark: &Self::Mark) -> io::Result<()> {
         self.source.restore(mark)
     }
+    #[inline(always)]
     fn skip(&mut self) -> io::Result<()> {
         self.source.skip()
     }
+    #[inline(always)]
     fn peek(&mut self) -> io::Result<u8> {
         self.source.peek()
     }
+    #[inline(always)]
     fn readbytes(&mut self, count: usize) -> io::Result<Cow<'de, [u8]>> {
         self.source.readbytes(count)
     }
+    #[inline(always)]
     fn readbytes_into(&mut self, bs: &mut [u8]) -> io::Result<()> {
         self.source.readbytes_into(bs)
     }
@@ -66,11 +71,13 @@ fn out_of_range<I: Into<BigInt>>(i: I) -> error::Error {
     error::Error::NumberOutOfRange(i.into())
 }
 
-impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: BinarySource<'de>> PackedReader<'de, 'src, D, N, Dec, S> {
+impl<'de, 'src, N: NestedValue, Dec: DomainDecode<N::Embedded>, S: BinarySource<'de>> PackedReader<'de, 'src, N, Dec, S> {
+    #[inline(always)]
     pub fn new(source: &'src mut S, decode_embedded: Dec) -> Self {
         PackedReader { source, decode_embedded, phantom: PhantomData }
     }
 
+    #[inline(always)]
     fn read(&mut self) -> io::Result<u8> {
         let v = self.peek()?;
         self.skip()?;
@@ -84,6 +91,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn varint(&mut self) -> io::Result<usize> {
         let mut shift = 0;
         let mut acc: usize = 0;
@@ -95,6 +103,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn peekend(&mut self) -> io::Result<bool> {
         if self.peek()? == Tag::End.into() {
             self.skip()?;
@@ -104,6 +113,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn peek_next_nonannotation_tag(&mut self) -> ReaderResult<Tag> {
         loop {
             match Tag::try_from(self.peek()?)? {
@@ -138,6 +148,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn read_signed_integer(&mut self, count: usize) -> io::Result<SignedInteger> {
         if count == 0 {
             return Ok(SignedInteger::from(0_i128));
@@ -174,6 +185,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn next_unsigned<T: FromPrimitive, F>(&mut self, f: F) -> ReaderResult<T>
     where
         F: FnOnce(u128) -> Option<T>
@@ -205,6 +217,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn next_signed<T: FromPrimitive, F>(&mut self, f: F) -> ReaderResult<T>
     where
         F: FnOnce(i128) -> Option<T>
@@ -258,9 +271,9 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
     }
 }
 
-impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: BinarySource<'de>>
-    Reader<'de, D, N>
-    for PackedReader<'de, 'src, D, N, Dec, S>
+impl<'de, 'src, N: NestedValue, Dec: DomainDecode<N::Embedded>, S: BinarySource<'de>>
+    Reader<'de, N>
+    for PackedReader<'de, 'src, N, Dec, S>
 {
     fn next(&mut self, read_annotations: bool) -> io::Result<Option<N>> {
         match self.peek() {
@@ -354,6 +367,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }))
     }
 
+    #[inline(always)]
     fn open_record(&mut self, arity: Option<usize>) -> ReaderResult<B::Type> {
         self.next_compound(Tag::Record, ExpectedKind::Record(arity))?;
         let mut b = B::Type::default();
@@ -361,6 +375,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         Ok(b)
     }
 
+    #[inline(always)]
     fn open_sequence_or_set(&mut self) -> ReaderResult<B::Item> {
         match self.peek_next_nonannotation_tag()? {
             Tag::Sequence => {
@@ -376,46 +391,54 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn open_sequence(&mut self) -> ReaderResult<()> {
         self.next_compound(Tag::Sequence, ExpectedKind::Sequence)
     }
 
+    #[inline(always)]
     fn open_set(&mut self) -> ReaderResult<()> {
         self.next_compound(Tag::Set, ExpectedKind::Set)
     }
 
+    #[inline(always)]
     fn open_dictionary(&mut self) -> ReaderResult<()> {
         self.next_compound(Tag::Dictionary, ExpectedKind::Dictionary)
     }
 
-    #[inline]
+    #[inline(always)]
     fn boundary(&mut self, _b: &B::Type) -> ReaderResult<()> {
         Ok(())
     }
 
+    #[inline(always)]
     fn close_compound(&mut self, _b: &mut B::Type, _i: &B::Item) -> ReaderResult<bool> {
         Ok(self.peekend()?)
     }
 
+    #[inline(always)]
     fn open_embedded(&mut self) -> ReaderResult<()> {
         self.next_compound(Tag::Embedded, ExpectedKind::Embedded)
     }
 
+    #[inline(always)]
     fn close_embedded(&mut self) -> ReaderResult<()> {
         Ok(())
     }
 
     type Mark = S::Mark;
 
+    #[inline(always)]
     fn mark(&mut self) -> io::Result<Self::Mark> {
         self.source.mark()
     }
 
+    #[inline(always)]
     fn restore(&mut self, mark: &Self::Mark) -> io::Result<()> {
         self.source.restore(mark)
     }
 
-    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<D, N>> {
+    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<N>> {
         loop {
             return Ok(match Tag::try_from(self.peek()?)? {
                 Tag::Embedded => {
@@ -452,7 +475,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
-    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<D, N>)> {
+    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<N>)> {
         match Tag::try_from(self.peek()?)? {
             Tag::Annotation => {
                 self.skip()?;
@@ -463,6 +486,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
         }
     }
 
+    #[inline(always)]
     fn next_boolean(&mut self) -> ReaderResult<bool> {
         match self.peek_next_nonannotation_tag()? {
             Tag::False => { self.skip()?; Ok(false) }
@@ -552,6 +576,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>, S: Binar
     }
 }
 
+#[inline(always)]
 fn decodestr(cow: Cow<'_, [u8]>) -> io::Result<Cow<'_, str>> {
     match cow {
         Cow::Borrowed(bs) =>
