@@ -268,6 +268,11 @@ impl Writer for BinaryOrderWriter {
         self.push(ptr);
         Ok(())
     }
+
+    #[inline(always)]
+    fn flush(&mut self) -> io::Result<()> {
+        Ok(())
+    }
 }
 
 macro_rules! fits_in_bytes {
@@ -522,4 +527,9 @@ impl<W: io::Write> Writer for PackedWriter<W>
         self.resume(ann);
         Ok(())
     }
+
+    #[inline(always)]
+    fn flush(&mut self) -> io::Result<()> {
+        self.0.flush()
+    }
 }
