@@ -1533,3 +1533,24 @@ impl<D: Embeddable> NestedValue for DummyValue<D> {
         self.0.1
     }
 }
+
+//---------------------------------------------------------------------------
+
+// https://stackoverflow.com/questions/34304593/counting-length-of-repetition-in-macro/34324856
+#[macro_export]
+//#[allow(unused_macros)]
+macro_rules! count__ {
+    () => (0usize);
+    ( $x:tt $($xs:tt)* ) => (1usize + $crate::count__!($($xs)*));
+}
+
+#[macro_export]
+macro_rules! rec {
+    ( $label:expr $(, $item:expr)* ) => {
+        {
+            let mut r__ = $crate::value::Value::record($label, $crate::count__!($($item)*));
+            $(r__.fields_vec_mut().push($item);)*
+            r__.finish().wrap()
+        }
+    }
+}
