@@ -1,5 +1,4 @@
 use crate::value::DomainEncode;
-use crate::value::Embeddable;
 use crate::value::IOValue;
 use crate::value::IOValueDomainCodec;
 use crate::value::NestedValue;
@@ -35,7 +34,7 @@ impl std::default::Default for CommaStyle {
 }
 
 impl TextWriter<&mut Vec<u8>> {
-    pub fn encode<D: Embeddable, N: NestedValue<D>, Enc: DomainEncode<D>>(
+    pub fn encode<N: NestedValue, Enc: DomainEncode<N::Embedded>>(
         enc: &mut Enc,
         v: &N,
     ) -> io::Result<String> {
@@ -122,6 +121,10 @@ impl<W: io::Write> TextWriter<W> {
             write!(self.w, " ")
         }
     }
+
+    pub fn borrow_write(&mut self) -> &mut W {
+        &mut self.w
+    }
 }
 
 impl<W: io::Write> CompoundWriter for TextWriter<W> {
@@ -314,4 +317,8 @@ impl<W: io::Write> Writer for TextWriter<W> {
         self.resume(ptr);
         Ok(())
     }
+
+    fn flush(&mut self) -> io::Result<()> {
+        self.w.flush()
+    }
 }
